package main

import (
	"encoding/json"
	"io/ioutil"
	"os"
	"strconv"
	"strings"

	sso "git.tcp.direct/kayos/tcpd-sso"
)

type OldUser struct {
	Username             string `json:"username"`
	Password             string `json:"password"`
	Name                 string `json:"name"`
	Storagebasedirectory string `json:"storagebasedirectory"`
	Storagenode          string `json:"storagenode"`
	Maildir              string `json:"maildir"`
	Quota                int    `json:"quota"`
	Domain               string `json:"domain"`
	Rank                 string `json:"rank"`
	Isadmin              int    `json:"isadmin"`
	Isglobaladmin        int    `json:"isglobaladmin"`
	Lastlogindate        string `json:"lastlogindate"`
	Lastloginipv4        int    `json:"lastloginipv4"`
	Passwordlastchange   string `json:"passwordlastchange"`
	Created              string `json:"created"`
	Modified             string `json:"modified"`
	Expired              string `json:"expired"`
	Active               int    `json:"active"`
}

var oldbytes []byte
var err error

var emailMap map[string]*sso.EmailAccount

func init() {
	if len(os.Args) == 1 {
		println("missing input json")
		os.Exit(1)
	}

	if oldbytes, err = ioutil.ReadFile(os.Args[1]); err != nil {
		println(err.Error())
		return
	}

	emailMap = make(map[string]*sso.EmailAccount)
}

func intToBool(i int) bool {
	switch i {
	case 0:
		return false
	case 1:
		return true
	default:
		panic(i)
	}
}

func contains(list []string, item string) bool {
	for _, el := range list {
		if item == el {
			return true
		}
	}
	return false
}

// GetUsersFromLegacy turns json data into sso.User objects.
func GetUsersFromLegacy(toconvert []byte) []*sso.User {
	var (
		oldUsers []OldUser
		allUsers []*sso.User
		allNames []string
		dupes    []string
		isDupe   bool
	)

	if err := json.Unmarshal(toconvert, &oldUsers); err != nil {
		println(err.Error())
		return nil
	}

	for _, old := range oldUsers {
		split := strings.Split(old.Username, "@")
		var uid = split[0]
		if contains(allNames, split[0]) {
			isDupe = true
			dupes = append(dupes, old.Username)
			for _, u := range allUsers {
				if u.UserID == split[0] {
					u.EmailAccounts = append(u.EmailAccounts, old.Username)
				}
			}
		} else {
			allNames = append(allNames, split[0])
		}

		if !isDupe {
			newuser := &sso.User{
				UserID:        uid,
				LegacyHash:    old.Password,
				Nickname:      old.Name,
				ContactEmail:  old.Username,
				GlobalAdmin:   intToBool(old.Isglobaladmin),
				EmailAccounts: []string{old.Username},
			}
			allUsers = append(allUsers, newuser)
		}

		if _, ok := emailMap[old.Username]; ok {
			panic(old.Username)
		}

		emailMap[old.Username] = &sso.EmailAccount{
			Domain:       split[1],
			FriendlyName: old.Name,
			Quota:        old.Quota,
			Type:         sso.MailDirStorage,
			DomainAdmin:  intToBool(old.Isadmin),
			MaildirDetails: sso.Maildir{
				StoreNode:          old.Storagenode,
				StoreBaseDirectory: old.Storagebasedirectory,
				StoreMaildir:       old.Maildir,
			},
		}
	}
	return allUsers
}

func main() {
	usrs := GetUsersFromLegacy(oldbytes)

	println("converted " + strconv.Itoa(len(usrs)) + " users successfully.\n\n")

	println("-- Users --\n")
	for _, u := range usrs {
		println ("------------------")
		println("ID: " + u.UserID)
		println("   emails: ")
		for _, addr := range u.EmailAccounts {
			print("      " + addr + " ")
			if emailAcct, ok := emailMap[addr]; !ok {
				print("EMAIL NOT FOUND! "+addr)
			} else {
				nm := emailAcct.FriendlyName
				if len(nm) > 0 {
					print(" (" + nm + ") ")
				}
			}
			print("\n")
		}
	}
	println ("------------------")
}
